pub enum ParseBase64Error {
InvalidByte {
offset: usize,
byte: u8,
},
InvalidLength {
length: usize,
},
InvalidLastSymbol {
offset: usize,
byte: u8,
},
InvalidPadding,
}Available on crate feature
base64 only.Expand description
Error returned by Base64Vec::from_str.
Variants
InvalidByte
An invalid byte was found at the given offset.
InvalidLength
The input length (in valid base64 symbols) is invalid.
InvalidLastSymbol
The last non-padding symbol has nonzero trailing bits that would be discarded, indicating corrupted or truncated input.
InvalidPadding
Padding was absent, incorrect, or otherwise not as expected.